## FFBS implementation for beta-vector
#  ffbas.R with the AR component


 #  Data Y , the time t is the second index (column)
 #  none of the other inputs are time-varying so far
 #  Also assume that transition G is the I matrix, saves time
 #         -- AR models maybe also handled faster w/out a matrix - ? 
 # 
 #   F will be X, the design matrix, and W will be Q. V is the variance of residuals, will be s2*C
 #
 # r is the vector of AR coefficients

 ffbs <- function(Y, F, V, W, C0, r){
   T <- dim(Y)[2]
   N <- dim(Y)[1]   ## will be Nsta
   p1 <- dim(F)[2]
   
   m <- matrix(0,p1,T)    # filtered mean
   a <- matrix(0,p1,T+1)    # forecast mean
   C <- array(0,c(p1,p1,T))   # filtered variances
   R <- array(0,c(p1,p1,T+1))   # forecast variances  
   sam <- matrix(0,p1,T)      # the sample
   I <- diag(rep(1,p1))
   G <- diag(r)              # system evolution matrix 
   
   a[,1]  <- 0
   R[,,1] <- C0 + W               # prior variance C0

   for (t in 1:T){
      
     et <- Y[,t] - F %*% a[,t]
     RtF <- R[,,t] %*% t(F)
     Qt <- F %*% RtF + V
     At <- RtF %*% solve(Qt)     # more efficient some other way?

     C[,,t] <- R[,,t] - RtF %*% t(At)
     m[,t] <- a[,t] + At %*% et

     a[,(t+1)] <- r * m[,t]                     # sic: elementwise product!
     R[,,(t+1)] <- G %*% C[,,t] %*% G  + W      # t(G) omitted

   }  # of Filtering cycle


   sam[,T] <- mvrnorm(1,m[,T],C[,,T])   

   for (t in (T-1):1){
     Bt <-  C[,,t] %*% G %*% solve(R[,,(t+1)]) 
     Ht <- C[,,t] %*% (I - G %*% t(Bt))     
     ht <- m[,t] + Bt %*% ( sam[,(t+1)] - a[,(t+1)])
     sam[,t] <- mvrnorm(1,m[,t],C[,,t])   
   }  # of Sampling cycle

   sam
}

 
#
#  will-be call:

#

#   Padj <- P - rep(1,Nsta) %*% t(as.matrix(allseas)) - rep(bsoi, Nsta) %*% t(as.matrix(soi))

#  
#   betasamp <- ffbs(Padj,X, C, diag( c( 1.2, 0.03, 0.03, 1, 0.1, 0.1 )^2), diag(rep(1,6)) )
							# Y, F, V, W, C0
#
#
#  
#
#  ffbscal(Y, 1, s2, q0^2, 1, r)


ffbscal <- function(Y, F, V, W, C0, r){
   T <- length(Y)
   p1 <- 1   

   m <- array(0,T)    # filtered mean
   a <- array(0,T+1)    # forecast mean
   C <- array(0,T)      # filtered variances
   R <- array(0,T+1)    # forecast variances  
   sam <- rep(NA,T)	# the sample
   
   a[1]  <- 0
   R[1] <- C0 + W               # prior variance C0

   for (t in 1:T){
      
     et <- Y[t] - F * a[t]
     RtF <- R[t] * t(F)
     Qt <- F * RtF + V
     At <- RtF / Qt

     C[t] <- R[t] - RtF * At
     m[t] <- a[t] + At *et

     a[(t+1)] <- r * m[t]                     # sic: elementwise product!
     R[(t+1)] <- r^2 * C[t]  + W      # t(G) omitted

   }  # of Filtering cycle


   sam[T] <- rnorm(1,m[T],sqrt(C[T]))   

   for (t in (T-1):1){
     Bt <-  C[t] * r /(R[(t+1)]) 
     Ht <- C[t] * (1 - r *Bt)     
     ht <- m[t] + Bt * ( sam[(t+1)] - a[(t+1)])
     sam[t] <- rnorm(1,m[t],sqrt(C[t])   )
   }  # of Sampling cycle

   sam
}

