##  Now with ffbs. Appears to give the same posterior mean as Matlab draft1 version
#                           when all parameters except betas were fixed
#  Uses ffbsar.R
#
#  
# 
#    version 2b cuts the number of data 
#    version 2c uses AR model
# 
#    version 3 uses "random effect" and attempts to extend the results of jsmind1.txt  --> somehow works! See July 29 bcup
#    version 3a 

source("ffbsar.R")     # run the ffbs function

MCout <- function(X, beg1, beg2, title){    # for a MCMC output X, plots the series, ACF and histogram
 opa <- par(mfrow=c(1,3))
 T <- length(X)
 beg1 <- min(beg1, T/2)
 beg2 <- min(beg2, T/2)
 plot(X[beg1:T],  main=title, type="l", ylab="")
 acf(X[beg2:T], main = "")    # paste("ACF of", title))
 hist(X[beg1:T], main ="", xlab=title)    # paste("Histogram of", title) )
}
    

 matrmean <- function(M){    # function equivalent to Matlab's columnwise mean
   ncol <- dim(M)[2];
   mm <- rep(0,ncol);
   for (j in 1:ncol){
       mm[j] <- mean(M[,j])
   } 
   mm
 }

 gee <- function(x){
   0.5*log(1-x^2) - bet[1]^2*(1-x^2)/(2*qbet[j]^2)
 }

 amean1 <-  function(X){        # input X should be in radians
   n <- dim(X)[2]
   msin <- matrmean(sin(X))
   mcos <- matrmean(cos(X))    ## computing the angle average for MFD, MG
   am <- atan(msin/mcos)
   
   for (i in 1:n){
     if (mcos[i] < 0) am[i] <- am[i] + pi  
   }
   (am %% (2*pi)) *180/pi    # output in degrees
 }
   

 require(MASS)   #    mvrnorm function 

  rd <- pi/180;

  # ----------------------------------------------------------------------------
  ##  0. Preprocessing
  
  Psta <- read.table("Psta.dat", na.strings="NaN")
  Nsta <- dim(Psta)[1]
 T <- dim(Psta)[2]-1         # for that particular dataset, 3 last months were all blank
 Nyear <- T/12
 Psta <- Psta[ ,1:(T+1)]
  
  soi34 <-  as.matrix(read.table("soi34.dat", na.strings="NaN"))
                           # values of the SOI index, aussie web site
  soi <- c(3.7, soi34[1,2:13])     # 3.7 is for Dec.69, lag -1 soi
  for (jj in 2:Nyear)
     soi <- c(soi, soi34[jj,2:13])

  soi <- soi[1:(length(soi)-1)]

  
  P <- as.matrix(Psta[,2:(T+1)])
  mean(P, na.rm = TRUE)
  
  # rows of P, change NaNs into naSub 
  naSub <- 99.99;
  naP <- is.na(P);
  
  for (i in 1:Nsta)
      P[i,] <- pmax(P[i,] , naP[i,]*naSub, na.rm=TRUE);
   
                    # save('stalist.txt','Stalist','-ASCII')
   
   stalist <- Psta[,1];
   ind <- order(stalist);    # indices of stalist entries, ordered from smallest to largest
  
   P <- sqrt(P[ind,]);   ## or may try an arbitrary Box-Cox transform
   Pmean <- matrmean(P);
   
   ## load covariates

    covjsm <- as.matrix(read.table("covjsm.csv", sep=",")); 
    covjsm[,2:4] <-  covjsm[,2:4]/1000;
    dall <- cbind(covjsm, P);         # all data together -- for monitoring purposes
  
   ## create X-matrix 
   
   aspid <- 3;   # corresponds to 5 km aspect
   asp <- covjsm[,4+aspid]*rd;
   X <- cbind(rep(1,Nsta), (covjsm[,3] - mean(covjsm[,3])),
              covjsm[,2] - mean(covjsm[,2]), covjsm[,4] - mean(covjsm[,4]),
	         cos(asp), sin(asp) );
   p1 <- dim(X)[2];    


   ptm <- proc.time()         ## eq. to Matlab's "tic"
    
   # ----------------------------------------------------------------------------
  ##  0. Initialize MCMC chain
  
  qbet <- c( 0.4, 0.03, 0.03, 0.8, 0.12, 0.12 );    # st.dev's of change in betas
  r <- rep(0.4, 6) 					# r is vector of AR coefficients 

  s2 <- 1;
  stau <- 0.85
  C <-  s2*diag(rep(1,Nsta)); 
  Cinv <- diag(rep(1,Nsta))/s2; 

  bsoi <- -0.015;    # coefficient in SOI regression. Not time-dependent
  soisq <- sum(soi^2)
  
 # s2arr <- rep(1,T);    ## variances for each t-regression
    
  beta <- matrix(1,p1,T);
  resarr <- P;          # array of residuals
  seas <- rep(0,12);
  
  M <- 20;           # number of MCMC runs saved, not counting skipped
  burnin <- 1;
  skip <- 1;
  
  bsoihist <- matrix(0,M,1);
  s2hist <- matrix(0,M,1);
  stauhist <- matrix(0,M,1);
  qbethist <- matrix (0,M,p1);
  betahist <- array(0,c(M,p1,T));
  rhist <- matrix(0, M, p1);
   MGhist <- matrix(0,M,T);
   MFDhist <- matrix(0,M,T);
   MGarr <- matrix(0,1,T);
   MFDarr <- matrix(0,1,T);
  One <- matrix(1, Nsta, Nsta);
   
  staeff <- matrix(0,Nsta,1);
  refhist <- matrix(0,M,1);      # random effect "tau" hyperparameter for staeff G_j

  Ntryref <- 0;
  reftau <- 200;
  
  ctr <- 0;

  for (mon in 1:12){ 				# moved seasonals here
          endmon <- (mon+12*(Nyear-1));
          smean <- mean(Pmean[seq(mon,endmon,12)]);
          seas[mon] <- smean;
  }
  allseas <- seas;
  for (jj in 2:Nyear)
	  allseas <- c(allseas, seas);
  
  # 
  # =========== Main Gibbs cycle ================================================================
  # 
  for (m in 1:(M*skip)){
  # 
  # 
   Q <- diag(qbet^2);   
    
  # ----------------------------------------------------------------------------
  ##  I.  Fitting the regression parameters, one by one. 
  
  Padj <- P - rep(1,Nsta) %*% t(as.matrix(allseas)) - rep(bsoi, Nsta) %*% t(as.matrix(soi))

  beta <- ffbs(Padj, X, C + (stau^2)*One, Q, diag(rep(100,6)), r)

 #> changed these
 #>>>>
    resid <- Padj - X %*% beta;     # these are not real residuals, contain MC error
    s2temp <- mean(resid^2);        
    
     # ----------------------------------
     #  processing MG and MFD
   
  for(t in 1:T){
      MFD <- atan(beta[6,t]/beta[5,t])/rd;
        if (beta[5,t] < 0)   MFD <- MFD + 180
      MFD <- MFD %% 360;
      MG <- atan(beta[3,t]/beta[2,t])/rd;
        if (beta[2,t] < 0)   MG <- MG + 180 
                     # !!! check these, something fishy about indices
      MG <- MG %% 360;
      MFDarr[t] <- MFD;
      MGarr[t] <- MG;        
   }   # of beta cycle
 #>>>>
 
    dfs <- (Nsta-1)*T
    s2 <- s2temp*dfs/rchisq(1,dfs)

    C <-  s2*diag(rep(1,Nsta)); 
    Cinv <- diag(rep(1,Nsta))/s2; 
  
  # ----------------------------------------------------------------------------
  ##  II. Seasonals
  
   resmean <- matrmean(resid);           # mean for every t
   resstmean <- t(matrmean(t(resid)));   # mean for every Station j  = y-bar_{\dot j}
   
    # 
    #  Just average of seasonals
    
   
  
  #   
  # ssd = sqrt(s2/Nyear);
  # for mon = 1:12; 
  #     endmon = (mon+12*(Nyear-1));
  #     smean = mean(resmean(mon:12:endmon));
  #     seas(mon) = seas(mon) + normrnd(smean,ssd);
  # end
  # seas(1) = 0;           ## to assure identifiability
  
  
   #  IIb. Fitting bsoi     and IIc. are temporarily taken out
   
     meansoi <- sum(soi*resmean)/soisq;   # I'm a little  uneasy about this -- looks like random walk
     sdbsoi <- sqrt((s2/Nsta + stau^2)/soisq);
     bsoi <- bsoi  + rnorm(1,meansoi, sdbsoi);    # cheated here before

  # ------ ----------------------------------------------------------------------
  ##  III. Fitting hyperparameters 
  
   # IIIa. Fitting Q. For now, independent gammas
   
   dfq <- T-1;
   pdfq <- c(10, 3, 3, 3, 3, 3);    # prior DF and mean for Gamma distr.
   pgam <- 0.1;
   
   for (j  in 1:p1){
       bet <- beta[j,];
       dfqj <- dfq + pdfq[j];
       ssq <- (sum((bet[1:(T-1)]*r[j] - bet[2:T])^2) + pgam*pdfq[j]);
       qbet[j] <- sqrt(ssq/rchisq(1,dfqj));
   }       
    

  # IIIb. Fitting stau    -- copied from jsmind1, where it was s2
   
   dfq <- T;
   pdfq <- 1;    # prior DF and mean for Gamma distr.
   pgam <- 0.1;
   dfs <- dfq + pdfq;
   
   ssq2 <- ( sum(resmean^2) + pgam*pdfq );
   stau <- sqrt(ssq2/rchisq(1,dfs))

   # IIIc. Fitting r    -- extended from scalar version in jsmind1  (Metropolis steps)   

   for (j in 1:p1){
    bet <- beta[j,];
    sbsq <- sum( (bet[1:(T-1)])^2)
    rhat <- sum(bet[1:(T-1)] * bet[2:T]) / sbsq;
    sdr <- qbet[j]/sqrt(sbsq);
    rprop <- max( min(rnorm(1, rhat, sdr) , 0.99), -0.99)
 
    pracc <- exp( min(gee(rprop) - gee(r[j])) )
    u <- runif(1)
     if (u < pracc)   r[j] <- rprop
  }
  


   # 
   #  save stuff
      
   if ((m %% skip) == 0){ 
      ctr <- ctr+1;
      bsoihist[ctr] <- bsoi; 
      qbethist[ctr,] <- qbet;
      betahist[ctr,,] <- beta;
      MGhist[ctr,] <- MGarr;
      MFDhist[ctr,] <- MFDarr;
      refhist[ctr] <- reftau;
      s2hist[ctr] <- sqrt(s2);
      stauhist[ctr] <- stau;
      rhist[ctr,] <- r;
   }
         
   
  }  # of MCMC run 
  

   print(proc.time() - ptm)

  # ------------------------------------------------------
  # == Post-processing
  
		
opar <- par(ask=interactive())
	mP <- matrmean(P);
	plot(mP[1:120],col="green",type="o")
	betaseas <- allseas + matrmean(betahist[,1,]); 
	 lines(betaseas[1:120])
  
 MCout(bsoihist, burnin, burnin, expression(beta[SOI]))  
 MCout(qbethist[,1],burnin, burnin,expression(q[0]) )
 MCout(qbethist[,4],burnin, burnin,expression(q[3]) )
 MCout(s2hist,burnin, burnin,expression(sigma) )
 MCout(stauhist,burnin, burnin,expression(sigma[tau]) )
 MCout(rhist[,4],burnin, burnin,expression(r[3]))
 par <- opar



par(mfrow=c(4,5)) 
for (i in 1:20)  
   plot(bsoihist, betahist[,1,i])  

for (i in 1:20)  
   plot(betahist[,1,20], betahist[,1,i])  




# =====================================
# =====================================
# =====================================   OUTPUT

 MGavg <- amean1(MGhist*rd)    
 hist(MGavg)

 MFDavg <- amean1(MFDhist*rd)  # computed using my MFD RF technique
 hist(MFDavg)

#  write.table(round(cbind(MGavg, MFDavg),2),"MGMFDavg.txt")
#  write.table(round(MGhist[,50:99],1),"MGhist.txt")
#  write.table(round(MFDhist[,50:99],1),"MFDhist.txt")


 
par(mfrow=c(4,5)) 
for (i in 101:120) 
     hist(MFDhist[,i], main= i)

for (i in 101:120) 
     hist(MGhist[,i], main= i)


## complete MCout graphs ==============================

for (i in 1:p1){
 MCout(qbethist[,i],burnin, burnin, paste(expression(q),i-1) )
}

for (i in 1:p1){
 MCout(rhist[,i],burnin, burnin,paste("r[",i-1,"]") )
}
for (i in 1:p1){
  plot(matrmean(betahist[,i,]), type="l", main=paste(expression(beta),"[",i-1,"]")        ); 
}  ## don't need them all, see below


  plot(mP, type="l", main=" ", xlab="Month", ylab=" " )
  titl0 <- paste(expression(beta),"[",0,"]") 
  titl3 <- paste(expression(beta),"[",3,"]") 
  plot(matrmean(betahist[,1,2:T]), type="l", main=expression(beta[0]), xlab="Month", ylab=" "        ); 
    plot(matrmean(betahist[,4,]), type="l", main= expression(beta[3]), xlab="Month", ylab=" "     ); 




# is there a seasonal relationship in beta_# indices?
 plot(rep(1:12,T/12), matrmean(betahist[,4,]), xlab="Month", ylab=expression(beta[3]))

## q-quantiles

 qout <- c(0.05, 0.25, 0.5, 0.75, 0.95)
 qarr <- NULL  

 for (i in 1:6){
  qarr <- rbind(qarr, quantile(qbethist[,i], qout))
}
round(qarr,3)

 


